package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.entity.JpCloudUser;
import com.jpsoft.picc.modules.base.service.CompanyUserService;
import com.jpsoft.picc.modules.base.service.JpCloudUserService;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.CompanyService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/company")
@Api(description = "企业信息")
public class CompanyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private JpCloudUserService jpCloudUserService;

    @ApiOperation(value="获取信息")
    @GetMapping("detail")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业编号", required = true, paramType = "query")
    })
    public MessageResult<Company> detail(String companyId){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(companyId);

            if (company != null) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

//    @ApiOperation(value="更新企业信息")
//    @PostMapping("update")
    @Deprecated
    public MessageResult<Company> update(@RequestBody Company company,@RequestAttribute String subject){
        MessageResult<Company> msgResult = new MessageResult<>();

        try {
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());
            
            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(company);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

//	@ApiOperation(value="删除")
//  @PostMapping("delete/{id}")
    @Deprecated
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Company company = companyService.get(id);
            company.setDelFlag(true);
            company.setUpdateBy(subject);
            company.setUpdateTime(new Date());

            int affectCount = companyService.update(company);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


//    @ApiOperation(value="批量删除")
//    @PostMapping("batchDelete")
    @Deprecated
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Company company = companyService.get(id);
                company.setDelFlag(true);
                company.setUpdateBy(subject);
                company.setUpdateTime(new Date());

                affectCount += companyService.update(company);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "企业名称", paramType = "form"),
            @ApiImplicitParam(name = "legalName",value = "法人代表", paramType = "form"),
            @ApiImplicitParam(name = "usccCode",value = "证件编号", paramType = "form"),
            @ApiImplicitParam(name = "subject",value = "当前用户编号（系统自动获取，不用填写）")
    })
    public MessageResult<Map> pageList(
            String name,String legalName,String usccCode,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        if (StringUtils.isNotEmpty(legalName)) {
            searchParams.put("legalName","%" + legalName + "%");
        }

        if (StringUtils.isNotEmpty(usccCode)) {
            searchParams.put("usccCode","%" + usccCode + "%");
        }

        Page<Company> page = companyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="获取企业人员信息")
    @RequestMapping(value = "companyUserDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyId",value = "企业ID", required = true, paramType = "query")
    })
    public MessageResult<CompanyUser> companyUserDetail(String companyId){
        MessageResult<CompanyUser> msgResult = new MessageResult<>();

        try {
            CompanyUser companyUser = companyUserService.findByCompanyId(companyId);

            //sz 查询用户真实姓名
            List<JpCloudUser> promoterList = jpCloudUserService.findUserByNames(Arrays.asList(companyUser.getUserName()));

            if(promoterList.size() > 0){
                JpCloudUser jpCloudUser = promoterList.get(0);
                companyUser.setRealName(jpCloudUser.getRealName());
                companyUser.setPhone(jpCloudUser.getPhone());
            }

            if (companyUser != null) {
                msgResult.setResult(true);
                msgResult.setData(companyUser);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取推广人员信息列表")
    @RequestMapping(value = "queryPromoterList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "promoters",value = "推广人员帐号(多个英文逗号隔开)", required = true, paramType = "query")
    })
    public MessageResult<List<JpCloudUser>> queryPromoterList(String promoters){
        MessageResult<List<JpCloudUser>> msgResult = new MessageResult<>();

        try {
            String[] promoterArray = promoters.split(",");
            List<String> list = new ArrayList<>();
            for (String promoter : promoterArray) {
                list.add(promoter);
            }

            List<JpCloudUser> jpCloudUserList = jpCloudUserService.findUserByNames(list);

            msgResult.setResult(true);
            msgResult.setData(jpCloudUserList);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;

    }


    @ApiOperation(value="获取推广人员信息列表")
    @RequestMapping(value = "promoterList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name",value = "姓名", paramType = "query"),
            @ApiImplicitParam(name = "phone",value = "电话", paramType = "query"),
            @ApiImplicitParam(name = "userName",value = "用户名", paramType = "query"),
            @ApiImplicitParam(name = "companyName",value = "企业名称", paramType = "query")
    })
    public MessageResult<Map> promoterList(
            String name,String phone,String userName,String companyName,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize
        ){

        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            //todo 先根据用户名、真实姓名、电话查询云平台
            List<String> promotionList = new ArrayList<>();

            if(StringUtils.isNotEmpty(userName) || StringUtils.isNotEmpty(name) || StringUtils.isNotEmpty(phone)) {
                List<JpCloudUser> jpCloudUserList = jpCloudUserService.findUser(userName, name, phone);

                if(jpCloudUserList!=null){
                    for (JpCloudUser jpCloudUser : jpCloudUserList) {
                        promotionList.add(jpCloudUser.getName());
                    }
                }

                if (promotionList.size()==0){
                    //查询为空
                    promotionList.add("null"); //不存在的用户名以便查询结果为0
                }
            }

            Map<String,Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","asc"));

            Page<Company> page = new Page<>();

            if(promotionList.size()>0) {
                searchParams.put("promotionList", promotionList);
            }

            if(StringUtils.isNotEmpty(companyName)){
                searchParams.put("name","%" + companyName + "%");
            }

            //todo 只读取有推广人的记录
            searchParams.put("hasPromotion",true);

            page = companyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

            List<String> userNames = page.stream()
                                    .map((company)->company.getPromoter())
                                    .filter((promoter)->StringUtils.isNotEmpty(promoter))
                                    .distinct()
                                    .collect(Collectors.toList());

            //todo 一次性读取本页推广人信息
            List<JpCloudUser> jpCloudUsers = jpCloudUserService.findUserByNames(userNames);

            if(jpCloudUsers!=null) {
                Map<String, JpCloudUser> userMap = new HashMap<>();

                for (JpCloudUser jpCloudUser : jpCloudUsers) {
                    userMap.put(jpCloudUser.getName(), jpCloudUser);
                }

                for (Company company : page) {
                    if (StringUtils.isNotEmpty(company.getPromoter())) {
                        company.setJpCloudUser(userMap.get(company.getPromoter()));
                    }
                    else{
                        company.setJpCloudUser(new JpCloudUser()); //以免前台做是否为空判断
                    }
                }
            }

            msgResult.setResult(true);
            msgResult.setData(PojoUtils.pageWrapper(page));
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;

    }
}
