package com.jpsoft.picc.modules.common.utils;

import com.itextpdf.text.*;
import com.itextpdf.text.pdf.*;
import lombok.extern.slf4j.Slf4j;

import java.io.FileOutputStream;

/**
 * @author 墨鱼_mo
 * @date 2020-2-5 13:30
 * 水印
 */
@Slf4j
public class Watermark extends PdfPageEventHelper {
    Font FONT = new Font(Font.FontFamily.HELVETICA, 30, Font.BOLD, new GrayColor(0.95f));
    private String waterCont;//水印内容

    public Watermark() {
    }

    public Watermark(String waterCont) {
        this.waterCont = waterCont;
    }

    @Override
    public void onEndPage(PdfWriter writer, Document document) {
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                ColumnText.showTextAligned(writer.getDirectContent(), Element.ALIGN_CENTER,
                        new Phrase(this.waterCont == null ? "PICC" : this.waterCont, FONT),
                        (50.5f + i * 250),
                        (40.0f + j * 150),
                        writer.getPageNumber() % 2 == 1 ? 45 : -45);
            }
        }
    }

    public static void imageWatermark(String inputPath, String outputPath,String images) throws Exception{

        try{
            PdfReader reader = new PdfReader(inputPath);
            PdfStamper stamper = new PdfStamper(reader, new FileOutputStream(outputPath));
            PdfGState gs1 = new PdfGState();
            gs1.setFillOpacity(0.25f);

            Image image = Image.getInstance(images);
         //   int imgHeight = (int)image.getHeight();
        //    int imgWidth = (int)image.getWidth();

            int n = reader.getNumberOfPages();
            PdfContentByte under;

            under = stamper.getOverContent(2);
            under.setGState(gs1);

            Rectangle pageRect = reader.getPageSizeWithRotation(2);

          //  image.setRotation(45);
            image.scaleToFit(pageRect.getWidth()/5,pageRect.getHeight()/5);

            image.setAbsolutePosition(pageRect.getWidth()/2,pageRect.getHeight()/5);
            under.addImage(image);



            for (int i = 4; i <= n; i++) {
                // 获得PDF最顶层
                under = stamper.getOverContent(i);
                under.setGState(gs1);

           //     Rectangle pageRect1 = reader.getPageSizeWithRotation(i);

                //A4像素 842*596
                float maxSize = 842;
             //   image.setRotation(45);
                image.scaleToFit(pageRect.getWidth()/5,pageRect.getHeight()/5);
                image.setAbsolutePosition(2*pageRect.getWidth()/3,4*pageRect.getHeight()/5);
                under.addImage(image);

//                for (int y = 0; y < Math.min(pageRect.getHeight(), maxSize); y += imgHeight)
//                {
//                    for (int x = 0; x < Math.min(pageRect.getWidth(), maxSize); x += imgWidth)
//                    {
//                        // 水印文字成45度角倾斜
////                        image.setRotation(30);// 旋转 弧度
//                        // 设置旋转角度
////                        image.setRotationDegrees(-45);// 旋转 角度
//                        // 设置等比缩放
////                        under.setColorFill(BaseColor.GRAY);
////                        image.scaleToFit(80,120);
//                        image.setRotation(45);
//                        image.setAbsolutePosition(x, y);
//                        under.addImage(image);
//                    }
//                }
            }
            stamper.close();
            reader.close();
        }catch (Exception e){
            log.error(e.getMessage(),e);
        }
    }


    public static void main(String[] args) {
        try {
            Watermark.imageWatermark("C:\\Users\\Administrator\\Desktop\\投保单.pdf","C:\\Users\\Administrator\\Desktop\\test.pdf","C:\\Users\\Administrator\\Desktop\\picc\\logo2.png");
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("done");

    }
}



