package com.jpsoft.picc.modules.common.utils;

import cn.hutool.json.JSONObject;
import com.jpsoft.picc.modules.common.config.WeixinConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
public class WeixinUtil {
    public static String getAccessToken(WeixinConfig config) throws Exception{
        String accessToken = "";

        StringBuilder urlBuilder = new StringBuilder();

        urlBuilder.append(config.getTokenUrl());

        urlBuilder.append("?").append("grant_type=").append("client_credential");
        urlBuilder.append("&").append("appid=").append(config.getAppId());
        urlBuilder.append("&").append("secret=").append(config.getAppSecret());

        HttpGet httpGet = new HttpGet(urlBuilder.toString());

        CloseableHttpClient httpClient = HttpClientBuilder.create().build();

        CloseableHttpResponse response = httpClient.execute(httpGet);

        HttpEntity entity = response.getEntity();

        String content = EntityUtils.toString(entity);

        JSONObject json = new JSONObject(content);

        if(json.containsKey("access_token")){
            accessToken = json.getStr("access_token");
        }

        return accessToken;
    }

    public static String createQrcode(WeixinConfig config,String text,long expireSeconds) throws Exception{
        String qrcodeUrl = "";

        String accessToken = getAccessToken(config);

        if (StringUtils.isEmpty(accessToken)){
            throw new Exception("获取基础token失败!");
        }

        StringBuilder urlBuilder = new StringBuilder();

        urlBuilder.append(config.getCreateQrCodeUrl())
                .append("?access_token=")
                .append(URLEncoder.encode(accessToken,"UTF-8"));

        HttpPost httpPost = new HttpPost(urlBuilder.toString());

        CloseableHttpClient httpClient = HttpClientBuilder.create().build();

        List<NameValuePair> list = new ArrayList<NameValuePair>();
        list.add(new BasicNameValuePair("expire_seconds", expireSeconds + ""));
        list.add(new BasicNameValuePair("action_name", "QR_STR_SCENE"));
        list.add(new BasicNameValuePair("scene_str", text));

        UrlEncodedFormEntity entity = new UrlEncodedFormEntity(list, "UTF-8");
        httpPost.setEntity(entity);

        HttpResponse res = httpClient.execute(httpPost);

        if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
            String content = EntityUtils.toString(res.getEntity());// 返回json格式：
            JSONObject ret = new JSONObject(content);

            if(ret.containsKey("result") && ret.getBool("result")){
                qrcodeUrl = ret.getStr("data");
            }
            else{
                log.error(ret.getStr("message"));
            }
        }

        return qrcodeUrl;
    }

    public static boolean sendTemplateMessage(WeixinConfig config,String toUserOpenId,String templateId,JSONObject data){
        boolean result = false;

        try {
            String accessToken = getAccessToken(config);

            if (StringUtils.isEmpty(accessToken)) {
                throw new Exception("获取基础token失败!");
            }

            StringBuilder urlBuilder = new StringBuilder();

            urlBuilder.append(config.getSendTmplMsgUrl())
                    .append("?access_token=")
                    .append(URLEncoder.encode(accessToken, "UTF-8"));

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("touser", toUserOpenId);
            jsonObject.put("template_id", templateId);
            jsonObject.put("data", data);

            StringEntity entity = new StringEntity(jsonObject.toString(), "utf-8");

            entity.setContentType("application/json");//发送json数据需要设置contentType

            HttpPost httpPost = new HttpPost(urlBuilder.toString());
            CloseableHttpClient httpClient = HttpClientBuilder.create().build();

            httpPost.setEntity(entity);

            HttpResponse res = httpClient.execute(httpPost);

            if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                String content = EntityUtils.toString(res.getEntity());
                JSONObject ret = new JSONObject(content);

                if (ret.getInt("errcode") == 0) {
                    result = true;
                } else {
                    log.error(content);
                }
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            result = false;
        }

        return result;
    }
}
