package com.jpsoft.picc.modules.pub.controller;

import cn.hutool.json.JSONObject;
import com.jpsoft.picc.config.JpCloudConfig;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.entity.Message;
import com.jpsoft.picc.modules.base.entity.TransactionRecord;
import com.jpsoft.picc.modules.base.service.*;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.SignUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.*;

/**
 * @author 墨鱼_mo
 * @date 2020-2-11 12:37
 */

@Controller
@RequestMapping("/pay")
@Api(description="支付接口")
@Slf4j
public class PayController {
    @Autowired
    private JpCloudConfig jpCloudConfig;

    @Autowired
    private TransactionRecordService transactionRecordService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private TemplateMessageService templateMessageService;

    @Autowired
    private MessageService messageService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @PostMapping(value = "/orderNotifyBack")
    @ResponseBody
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRED)
    public Map orderNotifyBack(String orderNo,String payType,
                               String code,String payTime,
                               String sign,String userMoney) {
        HashMap<String, Object> retMap = new HashMap<String, Object>();

        String status = "success";
        String message = "订单支付成功";
//        JSONObject body = new JSONObject();
        log.warn("payTime>>>" + payTime + "paymentType>>>" + payType + ">>>>orderNo>>" + orderNo + "sign>>>" + sign);

        try {
            if (StringUtils.isEmpty(orderNo)) {
                throw new Exception("订单号不存在");
            }

            HashMap<String, Object> signMap = new HashMap<String, Object>();

            signMap.put("orderNo", orderNo);
            signMap.put("payType", payType);
            signMap.put("code", code);
            signMap.put("payTime", payTime);
            signMap.put("userMoney", userMoney);

            boolean result = SignUtil.validateSign(signMap, jpCloudConfig.getPlatformKey(), sign);

            if (!result) {
                throw new Exception("签名不正确");
            }

            TransactionRecord transactionRecord = transactionRecordService.getByOrderNo(orderNo);

            if (transactionRecord == null) {
                throw new Exception("订单不存在");
            } else if ("20".equals(transactionRecord.getPaymentStatus())) {
                message = "订单已支付";
                status = "success";
            } else {
                InsuranceApplication insuranceApplication = insuranceApplicationService.get(transactionRecord.getApplicationId());

                transactionRecord.setPaymentStatus("20");
                transactionRecord.setPaymentTime(new Date());
                transactionRecord.setBuyType(payType);
                transactionRecord.setUpdateTime(new Date());

                transactionRecordService.update(transactionRecord);

                //将投保单状态改为待制单
                if (insuranceApplication != null) {
                    insuranceApplication.setStatus(PolicyStatus.PendingMakePolicy.getValue() + "");
                    insuranceApplication.setUpdateTime(new Date());

                    insuranceApplicationService.update(insuranceApplication);
                }

                InsurancePolicy firstPolicy = applicationPolicyService.findFirstPolicyByApplicationId(insuranceApplication.getId());

                DecimalFormat df = new DecimalFormat("0.00");

                String opinion = "投保单号：" + insuranceApplication.getPolicyNo() + "已缴费成功，共计："
                        + df.format(insuranceApplication.getInsuranceFee()) + "元。";

                //todo 记录；流程历史
                InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();
                insurancePolicyRecord.setId(UUID.randomUUID().toString());
                insurancePolicyRecord.setPolicyId(firstPolicy.getId());
                insurancePolicyRecord.setStatus(PolicyStatus.PendingPay.getValue() + "");
                insurancePolicyRecord.setOpinion(opinion);
                insurancePolicyRecord.setDelFlag(false);
                insurancePolicyRecord.setCreateTime(new Date());

                insurancePolicyRecordService.insert(insurancePolicyRecord);

                //todo 发送微信模板消息
                templateMessageService.sendTemplateMessage(firstPolicy.getId(),PolicyStatus.PendingPay,PolicyStatus.PendingMakePolicy);

                //todo 发送站内消息通知企业用户(正常缴费荆鹏云会发短信消息给企业用户)
                CompanyUser companyUser = companyUserService.findByCompanyId(insuranceApplication.getCompanyId());

                if (companyUser!=null){
                    Message siteMessage = new Message();

                    siteMessage.setId(UUID.randomUUID().toString());
                    siteMessage.setTitle("缴费成功通知");
                    siteMessage.setContent(opinion);

                    siteMessage.setCreateTime(new Date());
                    siteMessage.setRecipientId(companyUser.getId());
                    siteMessage.setDelFlag(false);

                    messageService.insert(siteMessage);
                }

                //todo 处理合并投保单(会将当前投保单设为假删除，放最后处理)
                processCombinePolicy(insuranceApplication,firstPolicy);

                message = "订单已支付";
                status = "success";
            }
        }
        catch(Exception ex){
            //在实际开发中，我们往往需要在方法中进行异常的捕获，从而对异常进行判断，为客户端返回提示信息。
            //但是此时由于异常的被捕获，导致事务的回滚没有被触发，导致事务的失败。
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();

            status = "fail";
            message = ex.getMessage();
        }

        retMap.put("status", status);
        retMap.put("message", message);

        return retMap;
    }

    private void processCombinePolicy(InsuranceApplication insuranceApplication,InsurancePolicy firstPolicy ) {
        if (firstPolicy != null) {
            //先更新状态为待制单
            firstPolicy.setStatus(PolicyStatus.PendingMakePolicy.getValue() + "");
            firstPolicy.setUpdateTime(new Date());

            insurancePolicyService.update(firstPolicy);
        }

        //todo 相同企业、相同险种、相同生效时间段 是否已经有投保单
//        List<InsurancePolicy> policyList = insurancePolicyService.findNextMonthPolicy(
//                insuranceApplication.getCompanyId(),
//                firstPolicy.getDefinitionId(),
//                firstPolicy.getStartTime(),
//                insuranceApplication.getEndTime(),
//                new String[]{PolicyStatus.Draft.getValue() + "",PolicyStatus.Back.getText() + ""});

//        for(InsurancePolicy policy : policyList) {
//            if(!policy.getId().equals(firstPolicy.getId())) {
//                //todo 将相同企业、相同险种、相同生效时间段的当月投保单取出，准备合并
//                curMonthPolicy = policyList.get(0);
//                break;
//            }
//        }

        InsurancePolicy combinePolicy = null;

        if(StringUtils.isNotEmpty(insuranceApplication.getCombinePolicyId())) {
            combinePolicy = insurancePolicyService.get(insuranceApplication.getCombinePolicyId());
        }

        if (combinePolicy != null) {
            //todo 关联参保人员
            List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(firstPolicy.getId());

            for (InsurancePolicyMember srcMember : memberList) {
                //人员不重复
                if (!insurancePolicyMemberService.exist(combinePolicy.getId(), srcMember.getMemberId())) {
                    InsurancePolicyMember destMember = new InsurancePolicyMember();

                    destMember.setId(UUID.randomUUID().toString());
                    destMember.setPolicyId(combinePolicy.getId());
                    destMember.setMemberId(srcMember.getMemberId());
                    destMember.setDelFlag(false);
                    destMember.setCreateTime(new Date());
                    destMember.setStatus(srcMember.getStatus());

                    insurancePolicyMemberService.insert(destMember);
                }
            }

            insurancePolicyService.updatePolicyNumAndAmount(combinePolicy, null);

            //todo 因为已合并，将当前投保申请单、当前每月投保单设为false
            insuranceApplication.setDelFlag(true);
            insuranceApplication.setUpdateTime(new Date());
            insuranceApplicationService.update(insuranceApplication);

            firstPolicy.setDelFlag(true);
            firstPolicy.setUpdateTime(new Date());
            insurancePolicyService.update(firstPolicy);
        }
    }

    @GetMapping(value="/paySuccess")
    @ApiOperation(value="返回前端")
    public String paySuccess(){
        return "redirect:/portal/";
    }



}
