package com.jpsoft.weixin.controller;

import com.github.pagehelper.Page;
import com.jpsoft.weixin.dto.MessageResult;
import com.jpsoft.weixin.dto.Sort;
import com.jpsoft.weixin.entity.ReplyMessage;
import com.jpsoft.weixin.service.ReplyMessageService;
import com.jpsoft.weixin.utils.PojoUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.*;

@RestController
@RequestMapping("/replyMessage")
@Api(description = "微信自动回复消息配置")
public class ReplyMessageController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ReplyMessageService replyMessageService;

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<ReplyMessage> add(ReplyMessage replyMessage){
        MessageResult<ReplyMessage> msgResult = new MessageResult<>();

        try {
            replyMessage.setCreateTime(new Date());

            int affectCount = replyMessageService.insert(replyMessage);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(replyMessage);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<ReplyMessage> edit(@PathVariable("id") String id){
        MessageResult<ReplyMessage> msgResult = new MessageResult<>();

        try {
            ReplyMessage replyMessage = replyMessageService.get(id);

            if (replyMessage != null) {
                msgResult.setResult(true);
                msgResult.setData(replyMessage);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<ReplyMessage> update(ReplyMessage replyMessage){
        MessageResult<ReplyMessage> msgResult = new MessageResult<>();

        try {
            replyMessage.setUpdateTime(new Date());
            
            int affectCount = replyMessageService.update(replyMessage);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(replyMessage);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<ReplyMessage> page = replyMessageService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtil.pageWrapper(page));

        return msgResult;
    }
}
