package com.jpsoft.weixin.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.json.JSONObject;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

public class WeixinUtil {

    public static boolean checkSignature(String token,String signature, String timestamp, String nonce) {
        String[] arr = new String[] { token, timestamp, nonce };

        String content = Arrays.stream(arr)
                .sorted(Comparator.naturalOrder())
                .collect(Collectors.joining());

        MessageDigest md = null;
        String tmpStr = null;
 
        try {
            md = MessageDigest.getInstance("SHA-1");
            // 将三个参数字符串拼接成一个字符串进行sha1加密
            byte[] digest = md.digest(content.getBytes("UTF-8"));

            StringBuffer hexstr = new StringBuffer();

            String shaHex = "";

            for (int i = 0; i < digest.length; i++) {
                shaHex = Integer.toHexString(digest[i] & 0xFF);
                if (shaHex.length() < 2) {
                    hexstr.append(0);
                }
                hexstr.append(shaHex);
            }

            tmpStr = hexstr.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }

        // 将sha1加密后的字符串可与signature对比，标识该请求来源于微信
        return signature.equals(tmpStr);
    }

    public static Map<String,Object> xmlToMap(InputStream inputStream) throws Exception {
        Map<String,Object> dataMap = new HashMap<>();

        // 创建sax解析工厂
        SAXParserFactory factory = SAXParserFactory.newInstance();
        // 创建sax转换工具
        SAXParser saxParser = factory.newSAXParser();

        saxParser.parse(inputStream,new DefaultHandler(){
            private String tagName;

            @Override
            public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
                tagName = qName;
            }

            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                tagName = null;
            }

            @Override
            public void characters(char[] ch, int start, int length) throws SAXException {
                if(tagName!=null){
                    String content = new String(ch,start,length);
                    dataMap.put(tagName,content);
                }
            }
        });

        return dataMap;
    }

    public static void replyTextMessage(HttpServletResponse response, String fromUserName, String toUserName, String content){
        StringBuilder sb = new StringBuilder();

        sb.append("<xml>");
        sb.append("<ToUserName><![CDATA[" + toUserName + "]]></ToUserName>");
        sb.append("<FromUserName><![CDATA["+ fromUserName + "]]></FromUserName>");
        sb.append("<CreateTime>" + DateTime.now().getTime() + "</CreateTime>");
        sb.append("<MsgType><![CDATA[text]]></MsgType>");
        sb.append("<Content><![CDATA[" + content + "]]></Content>");
        sb.append("</xml>");

        try {
            response.getWriter().print(sb.toString());
            response.getWriter().close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static JSONObject xmlToJson(ServletInputStream inputStream) throws Exception {
        JSONObject jsonObject = new JSONObject();

        // 创建sax解析工厂
        SAXParserFactory factory = SAXParserFactory.newInstance();
        // 创建sax转换工具
        SAXParser saxParser = factory.newSAXParser();

        saxParser.parse(inputStream,new DefaultHandler(){
            private String tagName;

            @Override
            public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
                tagName = qName;
            }

            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                tagName = null;
            }

            @Override
            public void characters(char[] ch, int start, int length) throws SAXException {
                if(tagName!=null){
                    String content = new String(ch,start,length);
                    jsonObject.put(tagName,content);
                }
            }
        });

        return jsonObject;
    }
}